/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.webapp.jonasadmin.cmi;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.management.Attribute;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.ow2.jonas.webapp.jonasadmin.JonasBaseAction;
import org.ow2.jonas.webapp.jonasadmin.JonasManagementRepr;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

public class ApplyCmiAction extends JonasBaseAction {

    @SuppressWarnings("unchecked")
    @Override
    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form, HttpServletRequest p_Request, HttpServletResponse p_Response) throws IOException, ServletException {
        String sDomainName = m_WhereAreYou.getCurrentDomainName();
        String serverName = m_WhereAreYou.getAdminJonasServerName();
        CmiForm oForm = (CmiForm) p_Form;
        String forwardName = "displayCmiInfo";
        String action = oForm.getAction();

        try {
            ObjectName cmiOn = new ObjectName(sDomainName+":type=protocol,name=CMIAdmin");
            if (JonasAdminJmx.hasMBeanName(cmiOn, serverName)) {
                if (action != null) {
                    if (action.equals("refresh")) {
                        Integer[] params = {new Integer(oForm.getDelayToRefresh())};
                        String[] signature = {"java.lang.Integer"};
                        JonasManagementRepr.invoke(cmiOn, "setDelayToRefresh", params, signature, serverName);

                    }
                }

                Set clusterList = (Set)JonasManagementRepr.getAttribute(cmiOn, "ClusterNames", serverName);
                List<ClusterAttribute> cmiClusterList = new ArrayList();
                String[] signature = {"java.lang.String"};
                for (Iterator iter = clusterList.iterator(); iter.hasNext();) {
                    String clusterName = (String) iter.next();
                    String[] params = {clusterName};
                    Set objectList = (Set)JonasManagementRepr.invoke(cmiOn, "getObjectNames", params, signature, serverName);
                    //Set objectList = (Set)cmiMbeansServer.invoke(cmiOn, "getObjectNames", params, signature);
                    cmiClusterList.add(new ClusterAttribute(clusterName, objectList));
                }
                oForm.setCmiClusterList(cmiClusterList);
                oForm.getProtocols().clear();
                Set<String> protocols = (Set)JonasManagementRepr.getAttribute(cmiOn, "Protocols", serverName);
                for (String proto : protocols) {
                    String[] params = {proto};
                    String protoUrl = JonasManagementRepr.invoke(cmiOn, "getJMXServiceURL", params, signature, serverName).toString();
                    oForm.getProtocols().add(new Attribute(proto, protoUrl));

                }
                oForm.setDelayToRefresh(((Integer)JonasManagementRepr.getAttribute(cmiOn, "DelayToRefresh", serverName)).intValue());
            }else {
                //cmi mbean is not registered
                p_Request.setAttribute("newcmi", false);
            }

        }catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward(forwardName));
    }

}
