/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.webapp.jonasadmin.cmi;

import java.io.IOException;
import java.util.List;
import java.util.Set;

import javax.management.Attribute;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.ow2.jonas.webapp.jonasadmin.JonasBaseAction;
import org.ow2.jonas.webapp.jonasadmin.JonasManagementRepr;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

public class CmiObjectAction extends JonasBaseAction {
     @SuppressWarnings("unchecked")
    @Override
        public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form, HttpServletRequest p_Request, HttpServletResponse p_Response) throws IOException, ServletException {
            String sDomainName = m_WhereAreYou.getCurrentDomainName();
            String serverName = m_WhereAreYou.getAdminJonasServerName();
            String forwardName = "displayCmiObjectInfo";
            String objName = p_Request.getParameter("objName");
            if (objName == null) {
                objName = m_WhereAreYou.getVar("objName");
            }else {
                m_WhereAreYou.addVar("objName", objName);
            }
            CmiObjectForm oForm = (CmiObjectForm) p_Form;
            String action = oForm.getAction();

            try {
                     ObjectName cmiOn = new ObjectName(sDomainName+":type=protocol,name=CMIAdmin");
                     if (JonasAdminJmx.hasMBeanName(cmiOn, serverName)) {
                     oForm.setName(objName);
                    // Set clusterList = (Set)JonasManagementRepr.invoke(cmiOn, "getClusterNames", null, null, serverName);
                     String[] signature = new String[5];
                     signature[0] = "java.lang.String";
                     String[] params = new String[5];
                     params[0] = objName;

                     if (action != null) {
                         if (action.equals("lbPolicy")) {
                             //change lbPolicy infos
                             signature[1] = "java.lang.String";
                             params[1] = oForm.getLbPolicyClass();
                             JonasManagementRepr.invoke(cmiOn, "setLBPolicyClassName", params, signature, serverName);
                             params[1] = oForm.getLbStrategy();
                             JonasManagementRepr.invoke(cmiOn, "setLBStrategyClassName", params, signature, serverName);
                         }
                     }
                     String[] parameters = {objName};
                     String[] sign = {"java.lang.String"};
                     oForm.setServerRefs((List)JonasManagementRepr.invoke(cmiOn, "getServerRefs", parameters, sign, serverName));
                     oForm.setLbPolicyClass((String)JonasManagementRepr.invoke(cmiOn, "getLBPolicyClassName", parameters, sign, serverName));
                     oForm.setLbStrategy((String)JonasManagementRepr.invoke(cmiOn, "getLBStrategyClassName", parameters, sign, serverName));
                     oForm.setMinPoolSize(((Integer)JonasManagementRepr.invoke(cmiOn, "getMinPoolSize", parameters, sign, serverName)).intValue());
                     oForm.setMaxPoolSize(((Integer)JonasManagementRepr.invoke(cmiOn, "getMaxPoolSize", parameters, sign, serverName)).intValue());
                     oForm.setItfName((String)JonasManagementRepr.invoke(cmiOn, "getItfName", parameters, sign, serverName));
                     Set<String> simpleLbProperties = (Set) JonasManagementRepr.invoke(cmiOn, "getPropertiesNamesForLBPolicy", parameters, sign, serverName);
                     oForm.getSimpleProperties().clear();
                     for (String property : simpleLbProperties) {
                        params[1] = property;
                        signature[1] = "java.lang.String";
                        String propValue = (String)JonasManagementRepr.invoke(cmiOn, "getPropertyForLBPolicy", params, signature, serverName);
                        oForm.getSimpleProperties().add(new Attribute(property, propValue));
                    }
                     oForm.setClusterName((String)JonasManagementRepr.invoke(cmiOn, "getClusterName", parameters, sign, serverName));
                     }else {
//                    	cmi mbean is not registered
                         p_Request.setAttribute("newcmi", false);
                    }
            }catch (Throwable t) {
                addGlobalError(t);
                saveErrors(p_Request, m_Errors);
                return (p_Mapping.findForward("Global Error"));
            }
            // Forward to the jsp.
            return (p_Mapping.findForward(forwardName));
        }
}
