/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.webapp.jonasadmin.domain;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.jonas.lib.management.javaee.J2eeObjectName;
import org.ow2.jonas.webapp.jonasadmin.JonasBaseAction;
import org.ow2.jonas.webapp.jonasadmin.JonasManagementRepr;
import org.ow2.jonas.webapp.jonasadmin.common.BeanComparator;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * Display domain and cluster informations.
 * @author Adriana Danes
 */
public class EditDomainAction extends JonasBaseAction {

    /**
     * Compute info about this domain or cluster.
     * Populate the DomainForm.
     * Eventually call the domain.jsp for display
     */
    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
            , HttpServletRequest p_Request, HttpServletResponse p_Response)
    throws IOException, ServletException {

        try {
            DomainForm oForm = (DomainForm) p_Form;
            String domainName = m_WhereAreYou.getCurrentDomainName();
            String currentServerName = m_WhereAreYou.getCurrentJonasServerName();
            ObjectName domainOn = J2eeObjectName.J2EEDomain(domainName);
            boolean changeMap;
            Boolean changeMapObj =  ((Boolean) m_Session.getAttribute("mapChanged"));
            if (changeMapObj == null) {
                changeMap = false;
            } else {
                changeMap = changeMapObj.booleanValue();
            }
            String selected = p_Request.getParameter("select");
            if (selected == null) {
                // By default we edit the domain page
                // -- this is necessary for example when creating a cluster in the
                // domain, at the end of the ApplyClusterAction we comme back
                // into EdirDomainAction
                // -- maybe this should be treated otherwise ?
                selected = domainName;
            }
            // domain or cluster name
            String name = selected;
            m_Session.setAttribute("currentCluster", name);

            // Support mgmt context switch from a slave to the master
            // (to the administrator server)
            String adminServerName = m_WhereAreYou.getAdminJonasServerName();
            boolean changeManagement = false;
            if (!adminServerName.equals(m_WhereAreYou.getCurrentJonasServerName())) {
                m_WhereAreYou.refreshServers(p_Request, domainName, adminServerName);
                // onCurrentServer = m_WhereAreYou.getCurrentJonasServer();
                changeManagement = true;
            }

            // Force the node selected in tree
            m_WhereAreYou.selectNameNode("domain", true);

            if (changeManagement) {
                refreshServerTree(p_Request);
            }
            if (changeMap) {
                //refreshDomainMonitoringTree(p_Request);
                refreshServerTree(p_Request);
                m_WhereAreYou.setTreeToRefresh(true);
                m_Session.setAttribute("mapChanged", new Boolean(false));
            }
            // domain or cluster
            boolean inCluster = false;
            if (domainName.equals(selected)) {
                // this is a domain
                oForm.setType("Domain");
            } else {
                inCluster = true;
                oForm.setType("Cluster");
            }

            // set general properties
            oForm.setName(name);
            oForm.setDomainName(domainName);
            //oForm.setDescription(getStringAttribute(onDomain, "description"));
            oForm.setCluster(inCluster);
            oForm.setMasterName(adminServerName);
            oForm.setMasterON(J2eeObjectName.J2EEServer(domainName, adminServerName).toString());
            oForm.setMaster(getBooleanAttribute(m_WhereAreYou.getCurrentDomain(), "master"));

            // get the servers list
            ArrayList al = new ArrayList();
            String[] servers = null;
            if (oForm.isMaster()) {
                servers = getServers(domainOn, name, adminServerName);
                oForm.setClusterType(getAttribute(domainOn, domainOn, "getClusterType", adminServerName));
            } else {
                servers = getStringArrayAttribute(domainOn, "serverNames");
            }
            for (int i = 0; i < servers.length; i++) {
                String serverName = servers[i];
                ObjectName on = J2eeObjectName.J2EEServer(domainName, serverName);
                ServerItem it = null;
                if (!oForm.isMaster() && serverName.equals(currentServerName)) {
                     it = new ServerItem(on, "RUNNING", null);

                } else {
                    String serverState = getAttribute(on, domainOn, "getServerState", adminServerName);
                    String clusterdName = getAttribute(on, domainOn, "getServerClusterdaemon", adminServerName);
                    it = new ServerItem(on, serverState, clusterdName);
                }
                al.add(it);
            }
            Collections.sort(al, new BeanComparator(new String[] {"name"}));
            p_Request.setAttribute("listServers", al);

            if (oForm.isMaster() && !inCluster) {
                // get the clusters list
                ArrayList clustAl = new ArrayList();
                String[] clusters = getStringArrayAttribute(domainOn, "clusters");
                for (int i = 0; i < clusters.length; i++) {
                    ObjectName on = ObjectName.getInstance(clusters[i]);
                    String clusterName = on.getKeyProperty("name");
                    if (!domainName.equals(clusterName)) {
                        String clusterState = getAttribute(on, domainOn, "getClusterState", adminServerName);
                        ServerItem it = new ServerItem(on, clusterState, null);
                        it.setName(clusterName);
                        clustAl.add(it);
                    }
                }
                Collections.sort(clustAl, new BeanComparator(new String[] {"name"}));
                p_Request.setAttribute("listClusters", clustAl);

                // get the cluster daemons list
                ArrayList clustDaemonAl = new ArrayList();
                String[] clusterDaemons = getStringArrayAttribute(domainOn, "clusterDaemons");
                for (int i = 0; i < clusterDaemons.length; i++) {
                    ObjectName on = ObjectName.getInstance(clusterDaemons[i]);
                    String cdName = on.getKeyProperty("name");
                    String clusterdaemonState = getAttribute(on, domainOn, "getClusterdaemonState", adminServerName);
                    ServerItem it = new ServerItem(on, clusterdaemonState, null);
                    it.setName(cdName);
                    clustDaemonAl.add(it);
                }
                Collections.sort(clustDaemonAl, new BeanComparator(new String[] {"name"}));
                p_Request.setAttribute("listCdps", clustDaemonAl);// Re-check server states for a more accurate display
                //dm.refreshStates();
            } else {
                p_Request.setAttribute("listClusters", new ArrayList());
                p_Request.setAttribute("listCdps", new ArrayList());
            }
        } catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward("Domain"));

    }

    private String getAttribute(ObjectName on, ObjectName domainOn, String attName, String serverName)
        throws MalformedObjectNameException {
        String value = null;
        String name = on.getKeyProperty("name");
        Object[] asParam = {name};
        String[] asSignature = {"java.lang.String"};
        value = (String) JonasManagementRepr.invoke(domainOn
                , attName, asParam, asSignature
                , serverName);
        return value;
    }

    private String[] getServers(ObjectName domainOn, String param, String currentServerName) {
        Object[] asParam = {param};
        String[] asSignature = {"java.lang.String"};
        return (String[]) JonasManagementRepr.invoke(domainOn
                , "getServersInCluster", asParam, asSignature
                , currentServerName);

    }
}
