/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.webapp.jonasadmin.service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.jonas.lib.util.JonasObjectName;
import org.ow2.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.ow2.jonas.webapp.jonasadmin.JonasBaseAction;
import org.ow2.jonas.webapp.jonasadmin.WhereAreYou;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;


/**
 * @author Michel-Ange ANTON
 */

public class ListServicesAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(DEPTH_SERVER) + WhereAreYou.NODE_SEPARATOR
            + "services", true);

        WhereAreYou oWhere = (WhereAreYou) p_Request.getSession().getAttribute(WhereAreYou.
                SESSION_NAME);
        String serverName = oWhere.getCurrentJonasServerName();
        String domainName = m_WhereAreYou.getCurrentDomainName();

        // no Form used
        try {
            ArrayList al = new ArrayList();
            // Detail Service
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.ear")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.earService(domainName), serverName)
                , "ActionListAppContainers"));
            al.add(new ServiceItem(m_Resources.getMessage(
                "treenode.jonas.server.services.ejbContainers")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.ejbService(domainName), serverName), "ActionListContainers"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.web")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.webContainerService(domainName), serverName)
                , "ActionListWebContainers"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.database")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.databaseService(domainName), serverName)
                , "ActionListDatabases"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.resource")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.resourceService(domainName), serverName)
                , "ActionListResourceAdapters"));
            al.add(new ServiceItem(m_Resources.getMessage(
                "treenode.jonas.server.services.transaction")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.transactionService(domainName), serverName)
                , "ActionEditJtmService"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.jms")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.jmsService(domainName), serverName), "ActionEditJmsService"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.mail")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.mailService(domainName), serverName)
                , "ActionEditMailService"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.security")
                , JonasAdminJmx.hasMBeanName(JonasObjectName.securityService(domainName), serverName)
                , "ActionEditServiceSecurity"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.discovery")
                    , JonasAdminJmx.hasMBeanName(JonasObjectName.discoveryService(domainName), serverName)
                    , "ActionEditServiceDiscovery"));
            al.add(new ServiceItem(m_Resources.getMessage("treenode.jonas.server.services.webservices")
                    , JonasAdminJmx.hasMBeanName(JonasObjectName.wsService(domainName), serverName)
                    , "ActionListWebService"));

            // sort
            Collections.sort(al, new ServiceItemByNameComparator());
            // Set list in the request
            p_Request.setAttribute("listServices", al);
        } catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward("Services"));
    }
}
