/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Polit??cnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 *
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.lib.ejb21;

import java.util.Stack;

import org.ow2.jonas.deployment.ejb.SessionDesc;
import org.ow2.jonas.lib.ejb21.ha.interceptor.HACurrentDelegateImpl;
import org.ow2.jonas.lib.util.Log;

import org.objectweb.carol.cmi.ha.RequestId;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Generic part of the EJBLocalHome implementation for replicated SFSBs
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public abstract class JRepStatefulLocalHome extends JSessionLocalHome {

    /**
     * Logger for traces
     */
    protected static Logger cmilogger = Log.getLogger("org.objectweb.carol.cmi.ha");

    /**
     * constructor
     * @param dd The Session Deployment Decriptor
     * @param bf The Session Factory
     * @throws Exception
     */
    public JRepStatefulLocalHome(SessionDesc dd, JSessionFactory bf) {
        super(dd, bf);
        if (cmilogger.isLoggable(BasicLevel.DEBUG)) {
            cmilogger.log(BasicLevel.DEBUG, "JRepStatefulLocalHome : this=" + this);
        }
    }

    /**
     * Called if we have to replicate a request
     * @param rctx The RequestCtx that was returned at preInvoke()
     */
    protected void replicateCreate(JStatefulSwitch sfsw) {
        if (cmilogger.isLoggable(BasicLevel.DEBUG)) {
            cmilogger.log(BasicLevel.DEBUG, "replicateBean : this=" + this);
        }
        // Create the rootId from the current HACtx/Thread association
        HACurrentDelegateImpl current = HACurrentDelegateImpl.getCurrent();
        // Obtain the root request
        Stack requests = current.getRequests();
        if (requests != null) {
            try {
                RequestId rootId = (RequestId) requests.get(0);
                JRepStatefulLocal jrsl = (JRepStatefulLocal) sfsw.getLocal();
                JRepUtil.addModifiedBean(sfsw, jrsl.getClusterOId(), rootId);
            } catch (ArrayIndexOutOfBoundsException e) {
                cmilogger.log(BasicLevel.DEBUG, "Not in a remote method call chain.");
                // do nothing
            }
        }
    }

    protected BeanFactory getBeanFactory() {
        return bf;
    }

    /**
     * Creates the EJBLocalObject This is in the generated class because it is
     * mainly "new objectClass()"
     * @return The Local Object
     */
    abstract public JSessionLocal createLocalObject();

}
