/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Politecnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.lib.ejb21.ha;

import javax.naming.Context;

import org.objectweb.carol.cmi.ha.ReplicationManager;
import org.objectweb.carol.cmi.ha.ReplicationManagerImpl;
import org.objectweb.carol.cmi.ha.interceptor.HACurrent;
import org.objectweb.carol.cmi.ha.interceptor.HACurrentDelegate;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.jonas.lib.ejb21.ha.interceptor.HACurrentDelegateImpl;
import org.ow2.jonas.lib.service.AbsServiceImpl;
import org.ow2.jonas.lib.util.Log;
import org.ow2.jonas.service.ServiceException;

/**
 * This class binds the HA service to Jonas and initializes the HA architecture.
 *
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 * @author benoit pelletier
 */
public class HaServiceImpl extends AbsServiceImpl implements HaService {

    /**
     * Logger.
     */
    private static final Logger logger = Log.getLogger("org.objectweb.carol.cmi.ha");

    private boolean started = false;

    private ReplicationManager replicationMgr;

    private HACurrentDelegate haCurrentDlg = null;

    public void doInit(final Context ctx) throws ServiceException {
        super.doInit(ctx);
        // TODO, Replication Manager will probably fails !
        try {
            replicationMgr = new ReplicationManagerImpl(ctx);
        } catch (Exception e) {
            throw new ServiceException("Error during HA Service startup");
        }
    }

    public void doStart() throws ServiceException {
        logger.log(BasicLevel.INFO, "Starting replication service...");
        try {


            // Init the HA current class
            haCurrentDlg = HACurrentDelegateImpl.getCurrent();
            HACurrent.getHACurrent().setDlg(haCurrentDlg);

            if (logger.isLoggable(BasicLevel.DEBUG)) {
                logger.log(BasicLevel.DEBUG, "replicationMgr=" + replicationMgr);
                logger.log(BasicLevel.DEBUG, "haCurrentDlg=" + haCurrentDlg);
            }

        } catch (Exception e) {
            throw new ServiceException("Error in GCL name");
        }

        // COMPLETE: Allow to start and shutdown service dynamically
        started = true;
        logger.log(BasicLevel.INFO, "Replication service started");
    }

    public void doStop() throws ServiceException {
        // COMPLETE: Allow to start and shutdown service dynamically
        if (started) {
            started = false;
            replicationMgr.clear(); // Turn off the GCL
            logger.log(BasicLevel.INFO, "Replication service stopped");
        }
    }

    public boolean isStarted() {
        return started;
    }

    /**
     * Return the replication manager
     */
    public ReplicationManager getReplicationManager() {

        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG, "replicationMgr=" + replicationMgr);
        }

        return replicationMgr;
    }
}
