/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.lib.jms;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

/**
 * Dummy Monolog Logger implementation.
 */
class DummyLogger extends LoggerImpl {

    /**
     * Dummy Logger used when jotm classes are used on client side
     */
    public void log(int level, Object o) {
    }
}


/**
 * Class used to log messages in Jms module.
 * @author Sebastien Chassande-Barrioz
 */
public class TraceJms {

    /**
     * JMS Logger.
     */
    public static Logger logger = new DummyLogger();

    /**
     * LoggerFactory used for JORAM.
     */
    public static LoggerFactory loggerfactory = null;

    /**
     * Configure JMS loggers.
     * @param lf The LoggerFactory
     */
    public static void configure(LoggerFactory lf) {
        loggerfactory = lf;
        logger = lf.getLogger("org.ow2.jonas.lib.jms");
    }

    /**
     * @return Returns <code>true</code> if JMS logger level is DEBUG.
     */
    public static boolean isDebug() {
        return (logger != null) && logger.isLoggable(BasicLevel.DEBUG);
    }

    /**
     * @return Returns the JOnAS Global LoggerFactory.
     */
    public static LoggerFactory getLoggerFactory() {
        return loggerfactory;
    }

}
