/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.lib.naming.osgi;

import javax.naming.spi.InitialContextFactoryBuilder;
import javax.naming.spi.NamingManager;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jonas.lib.naming.SingletonComponentContextFactory;
import org.ow2.jonas.lib.naming.SingletonNamingManager;
import org.ow2.jonas.naming.JComponentContextFactory;
import org.ow2.jonas.naming.JNamingManager;
import org.ow2.jonas.registry.RegistryService;

/**
 * JOnAS Naming 'Service' Activator.
 * @author Guillaume Sauthier
 * TODO Needs a better logging solution :)
 */
public class Activator implements BundleActivator, ServiceListener {

    /**
     * The shared {@link JComponentContextFactory} to register.
     */
    private JComponentContextFactory factory;

    /**
     * The shared {@link JNamingManager} to register.
     */
    private JNamingManager naming;

    private BundleContext bc;

    private ServiceRegistration srCC;

    private ServiceRegistration srNM;

    /**
     * Start this bundle.
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(BundleContext bundleCtx) throws Exception {
        System.out.println("Starting Naming Bundle...");
        bc = bundleCtx;

        // Register the JComponentContextFactory. This is needed for Carol.
        factory = SingletonComponentContextFactory.getInstance();
        srCC = bc.registerService(JComponentContextFactory.class.getName(), factory, null);

        // NamingManager can be registered only after carol is started, because it
        // uses InitialContext and thus trigger a carol initialization.
        String filter = "("+Constants.OBJECTCLASS+"="+RegistryService.class.getName()+")";
        bc.addServiceListener(this, filter);
    }

    private void registerNaming() throws Exception  {
        // Set our own InitialContextFactoryBuilder
        // Try to load ICF classes from various ClassLoaders
        if (!NamingManager.hasInitialContextFactoryBuilder()) {
            // Only set a new Builder if none was specified
            InitialContextFactoryBuilder builder = new OSGiInitialContextFactoryBuilder(bc);
            NamingManager.setInitialContextFactoryBuilder(builder);
        } // Else: maybe only log the fact

        // Register the JNamingManager
        naming = SingletonNamingManager.getInstance();
        srNM = bc.registerService(JNamingManager.class.getName(), naming, null);

    }

    /**
     * Default bundle stop.
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(BundleContext bc) throws Exception {
        System.out.println("Stopping Naming Bundle...");
        srNM.unregister();
        srCC.unregister();
    }

    public void serviceChanged(ServiceEvent serviceEvent) {
        ServiceReference sref = serviceEvent.getServiceReference();
        try {
            registerNaming();
        } catch (Exception e) {
            System.out.println("Cannot register Naming Service!");
        }
    }

}
