/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.lib.management.agent;

import java.util.List;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jonas.properties.ServerProperties;

/**
 * Jmx Agent Activator.
 * @author Philippe Durieux
 */
public class Activator implements BundleActivator {

    private MBeanServer mbeanserver = null;
    private ServiceRegistration serverRegistration;

    private String domain;

    /**
     * Start  Bundle.
     * @param bc OSGi {@link BundleContext}
     * @throws Exception when TS startup has failed.
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(final BundleContext bc) throws Exception {
        System.out.println("Starting jonas-jmx-agent Bundle...");

        // Get DomainName and ServerName from the Bootstrap service
        // TODO assume service already started.
        ServiceReference sr = bc.getServiceReference(ServerProperties.class.getName());
        final ServerProperties props = (ServerProperties) bc.getService(sr);
        domain = props.getDomainName();

        // Look if there is already a mbeanserver for jonas.
        List mbeanServers = MBeanServerFactory.findMBeanServer(null);
        if (mbeanServers.size() > 0) {
            mbeanserver = (MBeanServer) mbeanServers.get(0);
        } else {
            // Need to create an MBean server
            mbeanserver =  MBeanServerFactory.createMBeanServer(domain);
        }
        serverRegistration = bc.registerService(MBeanServer.class.getName(), mbeanserver, null);
   }

    /**
     * {@inheritDoc}
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(final BundleContext bc) throws Exception {
        System.out.println("Stopping jonas-jmx-agent Bundle...");
        serverRegistration.unregister();
    }

}
