/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.lib.management.agent.osgi;

import java.util.List;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jonas.lib.bootstrap.JProp;
import org.ow2.jonas.properties.ServerProperties;

/**
 * MBeanServer registration component.
 * @author Philippe Durieux
 * @author Francois Fornaciari TODO Needs a better logging solution
 */
public class MBeanServerRegistration {

    /**
     * MBeanServer.
     */
    private MBeanServer mbeanserver = null;

    /**
     * MBeanServer service registration.
     */
    private ServiceRegistration serverRegistration;

    /**
     * OSGi BundleContext.
     */
    private BundleContext bc = null;

    /**
     * ServerProperties service object
     */
    private ServerProperties serverProperties = null;

    /**
     * Constructor.
     * @param bc OSGi {@link BundleContext}.
     */
    public MBeanServerRegistration(BundleContext bc) {
        this.bc = bc;
    }

    /**
     * Register the MBeanServer as an OSGi service.
     */
    public void start() {
        // Domain name
        String domainName = serverProperties.getDomainName();

        // Look if there is already a MBeanServer for JOnAS.
        List mbeanServers = MBeanServerFactory.findMBeanServer(null);
        if (mbeanServers.size() > 0) {
            mbeanserver = (MBeanServer) mbeanServers.get(0);
        } else {
            // Need to create an MBean server
            mbeanserver = MBeanServerFactory.createMBeanServer(domainName);
        }
        serverRegistration = bc.registerService(MBeanServer.class.getName(), mbeanserver, null);
    }

    /**
     * Unregister the MBeanServer.
     */
    public void stop() {
        serverRegistration.unregister();
    }
}
