/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.deployer;

import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.api.deployer.DeployerException;
import org.ow2.util.ee.deploy.api.deployer.IDeployer;
import org.ow2.util.ee.deploy.api.deployer.IDeployerManager;
import org.ow2.util.ee.deploy.impl.deployer.DeployerManager;

/**
 * This service is in fact a singleton but the deployer manager should be
 * exposed as an OSGi service.
 * @author Florent BENOIT
 */
public class DeployerManagerService {

    /**
     * Unique instance of the Deployer Manager.
     */
    private static IDeployerManager deployerManager = null;

    /**
     * Register a new Deployer on this manager instance.
     * @param deployer the deployer to register
     */
    public static void register(final IDeployer deployer) {
        getDeployerManager().register(deployer);
    }

    /**
     * Unregister a Deployer from this manager instance.
     * @param deployer the given deployer to unregister.
     */
    public static void unregister(final IDeployer deployer) {
        getDeployerManager().unregister(deployer);
    }

    /**
     * Init the deployer manager if there is no previous manager.
     * @return the deployer manager
     */
    public static IDeployerManager getDeployerManager() {
        if (deployerManager == null) {
            deployerManager = new DeployerManager();
        }
        return deployerManager;
    }

    /**
     * Deploy a given deployable by finding the first deployer that accept this
     * type of deployable.
     * @param deployable the given deployable to deploy.
     * @throws DeployerException if it cannot be undeployed.
     */
    public static void deploy(IDeployable<?> deployable) throws DeployerException {
        getDeployerManager().deploy(deployable);
    }

    /**
     * Undeploy a given deployable by finding the first deployer that accept
     * this type of deployable.
     * @param deployable the given deployable to undeploy.
     * @throws DeployerException if it cannot be undeployed.
     */
    public static void undeploy(IDeployable<?> deployable) throws DeployerException {
        getDeployerManager().undeploy(deployable);
    }

    /**
     * Deploy a given deployable by finding the first deployer that accept this
     * type of deployable.
     * @param deployable the given deployable to deploy.
     * @throws DeployerException if it cannot be undeployed.
     */
    public static void isDeployed(IDeployable<?> deployable) throws DeployerException {
        getDeployerManager().isDeployed(deployable);
    }
}
