/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.registry.carol.osgi;

import java.io.File;
import java.net.URL;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.jonas.lib.bootstrap.JProp;
import org.ow2.jonas.lib.execution.ExecutionResult;
import org.ow2.jonas.lib.execution.IExecution;
import org.ow2.jonas.lib.execution.RunnableHelper;
import org.ow2.jonas.naming.JComponentContextFactory;
import org.ow2.jonas.naming.JComponentContextFactoryDelegate;
import org.ow2.jonas.properties.ServerProperties;
import org.ow2.jonas.registry.RegistryService;
import org.ow2.jonas.registry.carol.CarolRegistryService;
import org.ow2.jonas.registry.carol.delegate.ORBCCFDelegate;

/**
 * JOnAS Carol Registry Service Activator.
 * @author Guillaume Sauthier TODO Needs a better logging solution :)
 */
public class Activator implements BundleActivator {

    /**
     * The service instance.
     */
    private CarolRegistryService carol;

    /**
     * Start Carol Bundle.
     * @param bc OSGi {@link BundleContext}
     * @throws Exception when Carol startup has failed (registry already started, ...)
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(final BundleContext bc) throws Exception {
        // TODO log this !
        System.out.println("Starting Carol Bundle...");

        // Assume that the JComponentContextFactory instance is already bound
        // FIXME
        ServiceReference sr = bc.getServiceReference(JComponentContextFactory.class.getName());
        final JComponentContextFactory factory = (JComponentContextFactory) bc.getService(sr);

        // Get domainName and serverName from the Bootstrap service
        // TODO assume service already started.
        sr = bc.getServiceReference(ServerProperties.class.getName());
        final ServerProperties props = (ServerProperties) bc.getService(sr);

        // Find the carol.properties URL
        File conf = new File(JProp.getJonasBase(), "conf");
        final URL carolConfig = new File(conf, "carol.properties").toURL();

        // Execute Carol Startup in an execution block
        IExecution<CarolRegistryService> carolStarter = new IExecution<CarolRegistryService>() {

            public CarolRegistryService execute() throws Exception {
                carol = new CarolRegistryService();
                carol.setName("registry");

                // init
                carol.setServerProperties(props);
                carol.setConfigurationURL(carolConfig);
                carol.setServerProperties(props);
                // TODO Needs to be configurable
                carol.setMode(CarolRegistryService.DEFAULT_MODE);

                // dependencies
                carol.setComponentContextFactory(factory);

                // check & start
                carol.checkRequirements();
                carol.start();

                // return carol
                return carol;

            }
        };

        // Execute
        ExecutionResult<CarolRegistryService> result = null;
        result = RunnableHelper.execute(Activator.class.getClassLoader(), carolStarter);

        // Throw an Exception if needed
        if (result.hasException()) {
            throw result.getException();
        }

        // Store the ref
        carol = result.getResult();

        // Register the service
        bc.registerService(RegistryService.class.getName(), carol, null);

        // register the delegate too
        bc.registerService(JComponentContextFactoryDelegate.class.getName(), new ORBCCFDelegate(), null);
    }

    /**
     * {@inheritDoc}
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(final BundleContext bc) throws Exception {
        // TODO Log this !
        System.out.println("Stopping Carol Bundle...");
        carol.stop();
    }

}
