/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.dbm.internal.osgi;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.jonas.dbm.internal.JOnASDataBaseManagerService;
import org.ow2.jonas.jmx.JmxService;
import org.ow2.jonas.properties.ServerProperties;
import org.ow2.jonas.tm.TransactionService;

/**
 * JOnAS Dbm Service Activator.
 * @author Philippe Durieux
 */
public class Activator implements BundleActivator {

    /**
     * The service instance.
     */
    private JOnASDataBaseManagerService dbm;

    /**
     * Start Dbm Bundle.
     * @param bc OSGi {@link BundleContext}
     * @throws Exception
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(final BundleContext bc) throws Exception {
        System.out.println("Starting Dbm Bundle...");

        // Assume that the TransactionService instance is already bound
        ServiceReference sr = bc.getServiceReference(TransactionService.class.getName());
        final TransactionService ts = (TransactionService) bc.getService(sr);

        // Get Configuration.
        sr = bc.getServiceReference(ServerProperties.class.getName());
        final ServerProperties props = (ServerProperties) bc.getService(sr);

        // Assume JmxService is already there.
        sr = bc.getServiceReference(JmxService.class.getName());
        final JmxService jmx = (JmxService) bc.getService(sr);

        // Create and init the Service.
        // Don't need to register it in OSGi, since it provides only DataSources
        // via the JNDI registry.
        dbm = new JOnASDataBaseManagerService();
        dbm.setServerProperties(props);
        dbm.setJmxService(jmx);
        dbm.setTransactionService(ts);
    }

    /**
     * {@inheritDoc}
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(final BundleContext bc) throws Exception {
        System.out.println("Stopping Dbm Bundle...");
        dbm = null;
    }

}
