/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.jmx.internal.osgi;

import javax.management.MBeanServer;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jonas.jmx.JmxService;
import org.ow2.jonas.jmx.internal.JOnASJMXService;
import org.ow2.jonas.properties.ServerProperties;
import org.ow2.jonas.registry.RegistryService;

/**
 * JOnAS JMX Service Activator.
 * @author Philippe Durieux
 * TODO Needs a better logging solution :)
 */
public class Activator implements BundleActivator {

    /**
     * The service instance.
     */
    private JOnASJMXService jmx;

    private ServiceRegistration srJmx;


    /**
     * Start JOnAS JMX Service Bundle.
     * @param bc OSGi {@link BundleContext}
     * @throws Exception when TS startup has failed.
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(final BundleContext bc) throws Exception {
        System.out.println("Starting JOnAS JMX Service Bundle...");

        // Create the Service
        jmx = new JOnASJMXService();

        // Get Configuration.
        ServiceReference sr = bc.getServiceReference(ServerProperties.class.getName());
        final ServerProperties props = (ServerProperties) bc.getService(sr);

        // dependencies
        // Assume that the RegistryService instance is already bound
        sr = bc.getServiceReference(RegistryService.class.getName());
        final RegistryService registry = (RegistryService) bc.getService(sr);
        jmx.setRegistryService(registry);

        // Retrieve the mbeanServer
        sr = bc.getServiceReference(MBeanServer.class.getName());
        final MBeanServer mbeanserver = (MBeanServer) bc.getService(sr);
        jmx.setMBeanServer(mbeanserver);

        jmx.setServerProperties(props);

        // Check dependencies
        jmx.checkRequirements();

        //
        jmx.setOsgi(true);

        // Start the Service
        jmx.start();

        // Register the Service to OSGi registry
        srJmx = bc.registerService(JmxService.class.getName(), jmx, null);
    }

    /**
     * {@inheritDoc}
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(final BundleContext bc) throws Exception {
        System.out.println("Stopping JOnAS JMX Service Bundle...");

        srJmx.unregister();
        if (jmx != null) {
            jmx.stop();
        }
    }

}
