/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Florent BENOIT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.security.realm.factory;

//import java
import java.util.Hashtable;

//import javax
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.RefAddr;
import javax.naming.Reference;

import org.ow2.jonas.lib.util.JNDIUtils;


//import objectweb.util
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class provides an implementation of a JResource Memory factory for
 * managing users
 * @author Florent Benoit
 */
public class JResourceMemoryFactory extends JResourceFactory {

    /**
     * The Java type for which this factory knows how to create objects.
     */
    private static final String FACTORY_TYPE = "org.ow2.jonas.security.realm.factory.JResourceMemory";

    /**
     * Creates a javax.mail.Session object using the location or reference
     * information specified.
     * @param obj the possibly null object containing location or reference
     *        information that can be used in creating an object.
     * @param name the name of this object relative to nameCtx, or null if no
     *        name is specified.
     * @param nameCtx the context relative to which the name parameter is
     *        specified, or null if name is relative to the default initial
     *        context.
     * @param environment the possibly null environment that is used in creating
     *        the object.
     * @return a newly created javax.mail.Session object with the specific
     *         configuration; null if an object cannot be created.
     * @throws Exception if this object factory encountered an exception while
     *         attempting to create an object, and no other object factories are
     *         to be tried.
     */
    public Object getObjectInstance(Object obj, Name name, Context nameCtx, Hashtable environment) throws Exception {

        //Get the reference
        Reference ref = (Reference) obj;

        //Get the class name
        String clname = ref.getClassName();

        //Check the class name
        if (!ref.getClassName().equals(FACTORY_TYPE)) {
            getLogger().log(BasicLevel.ERROR, "Cannot create object : required type is '" + FACTORY_TYPE
                    + "', but found type is '" + clname + "'.");
            return null;
        }

        Hashtable users = new Hashtable();
        Hashtable groups = new Hashtable();
        Hashtable roles = new Hashtable();

        String jResName = (String) ref.get("name").getContent();

        RefAddr refAddr = null;
        refAddr = ref.get("users");
        if (refAddr != null) {
            users = (Hashtable) JNDIUtils.getObjectFromBytes((byte[]) refAddr.getContent(), getLogger());
        }

        refAddr = ref.get("groups");
        if (refAddr != null) {
            groups = (Hashtable) JNDIUtils.getObjectFromBytes((byte[]) refAddr.getContent(), getLogger());
        }

        refAddr = ref.get("roles");
        if (refAddr != null) {
            roles = (Hashtable) JNDIUtils.getObjectFromBytes((byte[]) refAddr.getContent(), getLogger());
        }

        // Create and return a new object
        JResourceMemory jResourceMemory = new JResourceMemory();
        jResourceMemory.setName(jResName);
        jResourceMemory.setUsers(users);
        jResourceMemory.setGroups(groups);
        jResourceMemory.setRoles(roles);

        return jResourceMemory;
    }

}