/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004-2005 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.ant.jonasbase;

import java.io.File;

import org.ow2.jonas.ant.JOnASBaseTask;

import org.apache.tools.ant.BuildException;


/**
 * Allow to configure the list of services in jonas.properties file
 * @author Florent Benoit
 */
public class Services extends JTask implements BaseTaskItf {

    /**
     * Info for the logger
     */
    private static final String INFO = "[JOnAS] ";

    /**
     * Services property
     */
    private static final String SERVICES_PROPERTY = "jonas.services";

    /**
     * List of services names
     */
    private String serviceNames = null;

    /**
     * Default constructor
     */
    public Services() {
        super();
    }

    /**
     * Set the names of the services
     * @param serviceNames list of services
     */
    public void setNames(String serviceNames) {
        this.serviceNames = serviceNames;
    }

    /**
     * Check the properties
     */
    private void checkProperties() {
        if (serviceNames == null) {
            throw new BuildException(INFO + "Property 'dataSource ' is missing.");
        }
    }

    /**
     * Execute this task
     */
    public void execute() {
        checkProperties();

        // Path to JONAS_BASE
        String jBaseConf = getDestDir().getPath() + File.separator + "conf";
        changeValueForKey(INFO, jBaseConf, JOnASBaseTask.JONAS_CONF_FILE, SERVICES_PROPERTY, serviceNames, false);

    }

}