/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.launcher.felix;

import java.io.File;
import java.util.Map;

import org.apache.felix.framework.Felix;
import org.apache.felix.framework.cache.BundleCache;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.ow2.jonas.launcher.felix.util.IOUtils;

/**
 * JOnAS Running on OSGi/Felix.
 * @author Guillaume Sauthier
 */
public class JOnAS {

    /**
     * OSGi Gateway instance.
     */
    private Felix felix;

    /**
     * Creates a new configured framework instance.
     * @param forceCleanUp true if cache directory must be deleted.
     * @throws Exception cannot create the internal gateway.
     */
    @SuppressWarnings("unchecked")
    public JOnAS(final boolean forceCleanUp) throws Exception {
        System.out.println("\nWelcome to OW2 JOnAS (Running on Felix).");
        System.out.println("   Type 'help' for available commands.");
        System.out.println("-----------------------------------------------\n");

        // Get a new provider
        IConfigurationProvider provider = getConfigurationProvider();

        // Retrieve the configuration
        Map config = provider.getConfiguration();

        // Delete the cache directory if requested
        if (forceCleanUp) {
            IOUtils.deleteDir(new File((String) config.get(BundleCache.CACHE_PROFILE_DIR_PROP)));
        }

        // Default System properties
     // if (System.getProperty("java.naming.factory.initial") == null) {
     //     System.setProperty("java.naming.factory.initial", "org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");
     //  }

        // Create the instance
        felix = new Felix(config, null);
    }

    /**
     * @return a new {@link IConfigurationProvider}.
     */
    protected IConfigurationProvider getConfigurationProvider() {
        // can be sub-classed
        return new DefaultConfigurationProvider();
    }

    /**
     * Start the OSGi framework.
     * @throws BundleException cannot start Felix
     */
    public void start() throws BundleException {
        felix.start();
    }

    /**
     * Stop the OSGi framework.
     * @throws BundleException cannot stop Felix
     */
    public void stop() throws BundleException {
        // Only stop if the system is still running
        if (felix.getState() == Bundle.ACTIVE) {
            felix.stop();
        }
    }

    /**
     * Start a new JOnAS.
     * @param args not used
     * @throws Exception if something failed
     */
    public static void main(final String[] args) throws Exception {

        // Creates the JOnAS
        // Check if the user wants to clean up the cache
        final JOnAS jonas = new JOnAS(Boolean.getBoolean("jonas.cache.clean"));

        // Start it !
        jonas.start();
        Runtime.getRuntime().addShutdownHook(new Thread() {

            public void run() {
                try {
                    jonas.stop();
                } catch (BundleException e) {
                    e.printStackTrace(System.err);
                }
            }
        });
    }

}
