/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.launcher.felix.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;

/**
 * Maven2 Utilities.
 * @author Guillaume Sauthier
 */
public final class Maven2Utils {

    /**
     * Private empty constructor.
     */
    private Maven2Utils() {
    }

    /**
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @return a string-ified URL
     * @throws FileNotFoundException if jar file is not in the repository.
     */
    public static String getBundleMaven2Location(final String groupId,
                                                 final String artifactId,
                                                 final String version) throws FileNotFoundException {

        // Assume that private Maven2 repository is in ~/.m2/repository
        String userHome = System.getProperty("user.home");
        File home = new File(userHome, ".m2".concat(File.separator).concat("repository"));

        // Creates a File from artifact info
        File bundle = new File(home, constructDirectoryPath(groupId, artifactId, version).concat(File.separator)
                                                   .concat(artifactId).concat("-").concat(version).concat(".jar"));

        // Check that the jar file exists
        if(!bundle.exists()) {
            throw new FileNotFoundException(bundle.getPath());
        }

        // transform into URL
        try {
            return bundle.toURL().toExternalForm();
        } catch (MalformedURLException e) {
            throw new FileNotFoundException(bundle.getPath());
        }
    }

    /**
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @return transform the artifact coordinates into a m2 repository path
     */
    private static String constructDirectoryPath(final String groupId,
                                                 final String artifactId,
                                                 final String version) {
        // Transform groupId . into /
        return groupId.replace('.', File.separatorChar).concat(File.separator).concat(artifactId)
                      .concat(File.separator).concat(version);
    }
}
